package it.unimi.di.archive4j;

/*
 * Copyright (C) 2008-2013 Alessio Orlandi and Sebastiano Vigna
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

import it.unimi.di.law.vector.ImmutableSparseVector;


/**
 * An Identity featurizer, that is, a featurizer that returns term counts as scores.
 * 
 * @author Alessio Orlandi
 */
public class IdentityFeaturizer<T extends DocumentSummary> extends AbstractFeaturizer<T, ImmutableSparseVector> {
	private static final long serialVersionUID = 1L;

	/** Global number of terms in the archive. */
	private final int totalTerms;

	public IdentityFeaturizer( final Archive<? extends T> archive ) {
		super( archive );
		this.totalTerms = archive.numberOfTerms();
	}

	@SuppressWarnings("unchecked")
	public ImmutableSparseVector get( final Object o ) {
		final T summary = (T)o;

		final int nTerms = summary.size();

		double[] scores = new double[ nTerms ];
		int[] terms = new int[ nTerms ];

		for ( int i = 0; i < nTerms; i++ ) {
			terms[ i ] = summary.term( i );
			scores[ i ] = summary.count( i );
		}

		return ImmutableSparseVector.getInstance( totalTerms, scores, terms, summary.id() );
	}
}
