package it.unimi.di.archive4j.scratch;


/**
 * Randomly checks a few WARC records against an archive by manually performing the whole process
 * manually ( that is, parsing ).
 * 
 * @author Alessio Orlandi
 */

public class WarcRandomCheck {

/*	private static final Logger LOGGER = LoggerFactory.getLogger( WarcRandomCheck.class );

	private static final MutableString word = new MutableString(), nonWord = new MutableString();

	private static final Int2IntOpenHashMap termcounter = new Int2IntOpenHashMap( 1000, Int2IntOpenHashMap.VERY_FAST_LOAD_FACTOR );

	@SuppressWarnings("unchecked")
	public static void main( String[] args ) throws Exception {
		final SimpleJSAP jsap = new SimpleJSAP( WarcRandomCheck.class.getName(), "Randomly checks an archive against a few WARC records.", new Parameter[] {
				new FlaggedOption( "indexedField", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, 'I', "indexedField", "Indexed field" ),
				new FlaggedOption( "urlMap", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.NOT_REQUIRED, 'u', "urlMap", "String map for graph urls" ),
				new FlaggedOption( "collection", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, 'c', "collection", "Warc document collection for the given archive." ),
				new FlaggedOption( "archive", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, 'a', "archive", "Archive basename." ),
				new FlaggedOption( "preprocess", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, 'p', "preprocess", "Preprocessing data basename." ),
				new Switch( "only2xx", '2', "only2xx", "Filter out all non-http-2xx records." ),
				new UnflaggedOption( "number", JSAP.INTEGER_PARSER, JSAP.REQUIRED, "Number of positions to test. When omitted, an explicit list of IDs (one-per-line) must be provided via stdin" ),
				new FlaggedOption( "seed", JSAP.INTEGER_PARSER, "0", JSAP.NOT_REQUIRED, 's', "seed", "Seed for PRNG used to generate ids." ),

		} );

		final JSAPResult jsapResult = jsap.parse( args );

		if ( jsap.messagePrinted() ) return;

		final WarcDocumentCollection collection = (WarcDocumentCollection)BinIO.loadObject( jsapResult.getString( "collection" ) );

		final StringMap smap = (StringMap)BinIO.loadObject( jsapResult.getString( "preprocess" ) + MergePreprocessedData.TERMMAP_EXTENSION );
		final Properties properties = new Properties( jsapResult.getString( "preprocess" ) + Preprocess.PROPERTIES_EXTENSION );

		final Archive<? extends ArrayDocumentSummary> archive = (Archive<? extends ArrayDocumentSummary>)ArchiveLoader.getInstance( jsapResult.getString( "archive" ) );
		if ( !archive.hasRandomAccess() ) throw new IllegalArgumentException( "Archive must be randomly accessed." );
		if ( archive.numberOfDocuments() == 0 ) {
			LOGGER.info( "Archive is empty. Finishing. " );
			return;
		}


		final IntSortedSet ids = new IntAVLTreeSet();

		int size = jsapResult.getInt( "number" );
		final MersenneTwister prng = new MersenneTwister( jsapResult.getInt( "seed" ) );

		LOGGER.info( "Generating " + size + " random numbers.. " );
		final int collectionSize = collection.size();
		while ( ids.size() < size )
			ids.add( Math.abs( prng.nextInt() ) % collectionSize );

		final TermProcessor tproc = ObjectParser.fromSpec( properties.getString( Preprocess.PropertyKeys.TERMPROCESSOR ), TermProcessor.class, MG4JClassParser.PACKAGE, new String[] { "getInstance" } );
		final String fieldName = properties.getString( Preprocess.PropertyKeys.FIELD );

		final ProgressLogger pl = new ProgressLogger( LOGGER );
		final boolean only2xx = jsapResult.getBoolean( "only2xx" );

		pl.expectedUpdates = ids.size();
		pl.itemsName = "documents";
		pl.displayFreeMemory = false;
		pl.start( "Analyzing " + ( only2xx ? "HTTP/2xx document only from the whole " : "" ) + ids.size() + " test set, considering field " + fieldName );

		final int[] result = run( ids, collection, archive, smap, tproc, jsapResult.userSpecified( "urlMap" ) ? (StringMap<CharSequence>)BinIO.loadObject( jsapResult.getString( "urlMap" ) ) : null,
				fieldName, only2xx, pl );

		pl.stop( "Finished with " + result[ 0 ] + " wrong items from the archive and " + result[ 1 ] + " items not found into archive." );

	}



	@SuppressWarnings("boxing")
	protected static int[] run( IntCollection ids, WarcDocumentCollection collection, Archive<? extends ArrayDocumentSummary> archive, StringMap<CharSequence> smap, TermProcessor processor,
			StringMap<CharSequence> urlMap, String fieldName, boolean only2xx, ProgressLogger pl ) throws IOException {

		final int field = collection.factory().fieldIndex( fieldName );

		int missingSummaries = 0;
		int wrongSummaries = 0;

		int idx;
		IntIterator iter;
		Document warcDocument = null;
		
		for ( iter = ids.iterator(), idx = iter.nextInt(); iter.hasNext(); idx = iter.nextInt() ) {
			final String idxString = "[" + idx + "]";

			if ( pl != null ) pl.lightUpdate();
			if ( warcDocument != null ) warcDocument.close();

			warcDocument = collection.document( idx );
			if ( warcDocument == null || warcDocument.uri() == null ) {
				LOGGER.debug( idxString + " appears to miss from WARC or have null URI. Skipping." );		
				continue;
			}

			final Reference2ObjectMap<Enum<?>, Object> metadata = collection.metadata( idx );
			final int httpStatus = metadata.containsKey( WarcDocumentSequence.MetadataKeys.HTTPCODE ) ? (Integer)metadata.get( WarcDocumentSequence.MetadataKeys.HTTPCODE ) : -1;

			if ( only2xx && httpStatus / 100 != 2 ) {
				LOGGER.debug( idxString + " has forbidden HTTP status " + httpStatus + ". Skipping." );
				continue;
			}

			final int archiveId = ( urlMap != null ? (int)urlMap.getLong( warcDocument.uri() ) : idx );
			if ( archiveId < 0 ) {
				LOGGER.debug( idxString + " URL " + warcDocument.uri() + " cannot be found in the url map. Skipping." );
				continue;
			}

			final ArrayDocumentSummary fromArchive = archive.getDocumentById( archiveId );
			if ( fromArchive == null ) {
				LOGGER.info( idxString + " is mapped via uri " + warcDocument.uri() + " to ID " + archiveId + " appears to miss from archive, skipping." );
				missingSummaries++;
				continue;
			}

			final ArrayDocumentSummary fromWarc = parse( archiveId, warcDocument, field, processor, smap );
			fromWarc.sort();
			fromArchive.sort();

			if ( !fromWarc.equals( fromArchive ) ) {
				LOGGER.error( "@" + idxString + " (URI: " + warcDocument.uri() + "): WARC version\n" + fromWarc + "\n mismatches with archive\n" + fromArchive );
				wrongSummaries++;
			}
			else LOGGER.debug( idxString + ": succesfully matched " + fromArchive.size() + " terms from " + fromArchive.length() + " words" );
		}

		collection.close();

		return new int[] { wrongSummaries, missingSummaries };
	}


	private static ArrayDocumentSummary parse( int id, Document document, int field, TermProcessor processor, StringMap<CharSequence> terms ) throws IOException {
		final WordReader wordReader = document.wordReader( field );
		final MutableString word = WarcRandomCheck.word;
		final MutableString nonWord = WarcRandomCheck.nonWord;
		final Int2IntOpenHashMap termcounter = WarcRandomCheck.termcounter;

		wordReader.setReader( (Reader)document.content( field ) );
		termcounter.clear();

		int len = 0;
		while ( wordReader.next( word, nonWord ) ) {
			if ( word.length() == 0 ) continue;
			len++;

			if ( !processor.processTerm( word ) ) continue;

			final int termNumber = (int)terms.getLong( word );

			if ( termNumber >= 0 ) {
				termcounter.put( termNumber, termcounter.get( termNumber ) + 1 );
			}
		}

		final int[] termData = new int[ termcounter.size() ];
		final int[] countData = new int[ termcounter.size() ];

		Int2IntMap.Entry e;
		int j = 0;
		final ObjectIterator<Int2IntMap.Entry> iterator = termcounter.int2IntEntrySet().fastIterator();

		while ( iterator.hasNext() ) {
			e = iterator.next();
			termData[ j ] = e.getIntKey();
			countData[ j++ ] = e.getIntValue();
		}

		return new ArrayDocumentSummary( termData, countData, id, null, len );
	}
*/
}
