package it.unimi.di.archive4j;

/*		 
 * Copyright (C) 2008-2013 Alessio Orlandi and Sebastiano Vigna 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

import it.unimi.di.archive4j.Archive;
import it.unimi.di.archive4j.ArchiveLoader;
import it.unimi.di.archive4j.ArrayDocumentSummary;
import it.unimi.di.archive4j.BitstreamArchiveWriter;
import it.unimi.di.archive4j.SequentialBitstreamArchive.CompressionFlags;
import it.unimi.di.archive4j.SequentialBitstreamArchive.CompressionFlags.Coding;
import it.unimi.di.archive4j.SequentialBitstreamArchive.CompressionFlags.Component;
import it.unimi.dsi.fastutil.objects.Object2ObjectOpenHashMap;

import java.io.File;

import junit.framework.TestCase;




/** @author Alessio Orlandi */
public class SequentialBitstreamArchiveTest extends TestCase {

	protected String basename;
	protected String basenameNoCounts;
	protected String basenameNoDocLengths;

	protected boolean notTearDown;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		final Object2ObjectOpenHashMap<Component, Coding> flags = new Object2ObjectOpenHashMap<Component, Coding>( CompressionFlags.DEFAULT );
		basename = TestCollection.buildUsualSequentialArchive( flags );
		flags.put( Component.COUNTS, null );
		basenameNoCounts = TestCollection.buildUsualSequentialArchive( flags );
		flags.putAll( CompressionFlags.DEFAULT );
		flags.put( Component.DOCLENGTHS, null );
		basenameNoDocLengths = TestCollection.buildUsualSequentialArchive( flags );
	}

	@Override
	protected void tearDown() throws Exception {

		if ( !notTearDown ) {
			TestCollection.cleanupBitstreamArchive( basename );
			TestCollection.cleanupBitstreamArchive( basenameNoCounts );
			TestCollection.cleanupBitstreamArchive( basenameNoDocLengths );
		}

		super.tearDown();
	}

	// TODO ArrayDocumentSummaryTest should contain this
	public void testParse() throws Exception {
		for ( String s : TestCollection.testDoc )
			assertEquals( s, ArrayDocumentSummary.parse( s, null ).toString() );
	}


	@SuppressWarnings("unchecked")
	public void testEmptyArchive() throws Exception {
		String localBasename = File.createTempFile( "emptyarch", null ).getAbsolutePath();

		int[] f = new int[ 0 ];

		final BitstreamArchiveWriter<ArrayDocumentSummary> writer = new BitstreamArchiveWriter<ArrayDocumentSummary>( localBasename, f, false, CompressionFlags.DEFAULT );
		writer.close();

		for( boolean b: new boolean[] { true, false } ) {
			Archive<ArrayDocumentSummary> archive = (Archive<ArrayDocumentSummary>)ArchiveLoader.getInstance( localBasename, b );

			assertEquals( 0, archive.numberOfDocuments() );
			assertEquals( 0, archive.numberOfTerms() );
			assertEquals( 0, archive.numberOfWords() );

			assertFalse( archive.iterator().hasNext() );

		}

		for ( String i : TestCollection.extensions ) new File( localBasename + i ).delete();

		new File( localBasename ).delete();
	}

	@SuppressWarnings("unchecked")
	public void testIterator() throws Exception {
		for( String s: new String[] { basename, basenameNoCounts, basenameNoDocLengths } ) 
			for( boolean b: new boolean[] { true, false } ) {
				try {
					Archive<ArrayDocumentSummary> archive = (Archive<ArrayDocumentSummary>)ArchiveLoader.getInstance( s, b );

					assertEquals( TestCollection.testCollection1.length, archive.numberOfDocuments() );
					assertEquals( TestCollection.numberOfWords1, archive.numberOfWords() );
					assertEquals( TestCollection.frequency1.length, archive.numberOfTerms() );

					// for ( int i = 0; i < TestCollection.frequency1.length; i++ )
					// assertEquals( TestCollection.frequency1[ i ], archive.frequency( i ) );

					int i = 0;
					for ( ArrayDocumentSummary doc : archive ) {
						assertEquals( TestCollection.testCollection1[ i ].size(), doc.size() );
						if ( doc.length() != -1 ) assertEquals( TestCollection.testCollection1[ i ].length(), doc.length() );
						doc.sort();
						for( int t = doc.size(); t-- != 0; ) {
							assertEquals( TestCollection.testCollection1[ i ].term( t ), doc.term( t ) );
							if ( doc.count( t ) != -1 ) assertEquals( TestCollection.testCollection1[ i ].count( t ), doc.count( t ) );
						}
						i++;
					}

				}
				catch ( Exception e ) {
					notTearDown = true;
					throw ( e );
				}
			}
	}
}
