package it.unimi.di.archive4j;


import it.unimi.di.archive4j.Archive;
import it.unimi.di.archive4j.ArchiveLoader;
import it.unimi.di.archive4j.ArrayDocumentSummary;
import it.unimi.di.archive4j.SequentialBitstreamArchive.CompressionFlags;
import it.unimi.di.archive4j.Subarchive;
import it.unimi.di.archive4j.Subarchive.SubarchiveWriter;

import java.io.File;
import java.util.Iterator;

import junit.framework.TestCase;


public class SubarchiveTest extends TestCase {
	String random;


	public void setUp() throws Exception {
		random = TestCollection.buildRandomAccessArchive( CompressionFlags.DEFAULT );
	}

	public void tearDown() throws Exception {
		TestCollection.cleanupBitstreamArchive( random );
	}


	@SuppressWarnings("unchecked")
	public void testSubarchive() throws Exception {
		final String test = File.createTempFile( "subarch", null ).getAbsolutePath();
		final Archive a = ArchiveLoader.getInstance( random );

		assertTrue( a.hasRandomAccess() );

		final SubarchiveWriter writer = new SubarchiveWriter( test, random, true );

		writer.append( a.getDocumentById( 4 ) );
		writer.append( a.getDocumentById( 129 ) );
		writer.append( a.getDocumentById( 10 ) );

		writer.close();

		final Subarchive<ArrayDocumentSummary> sub = (Subarchive<ArrayDocumentSummary>)ArchiveLoader.getInstance( test );
		final int trueFreqs[] = { 1, 1, 0, 0, 1, 0, 1 };


		assertEquals( 3, sub.numberOfDocuments() );
		assertEquals( a.numberOfTerms(), sub.numberOfTerms() );
		assertEquals( 109, sub.numberOfWords() );
		for ( int i = 0; i < trueFreqs.length; i++ )
			assertEquals( trueFreqs[ i ], sub.frequency( i ) );

		assertEquals( a.getDocumentById( 10 ), sub.getDocumentById( 10 ) );
		assertEquals( a.getDocumentById( 4 ), sub.getDocumentById( 4 ) );
		assertEquals( a.getDocumentById( 129 ), sub.getDocumentById( 129 ) );

		assertNull( sub.getDocumentById( 11 ) );

		final Iterator<ArrayDocumentSummary> iter = sub.iterator();

		assertEquals( a.getDocumentById( 4 ), iter.next() );
		assertEquals( a.getDocumentById( 10 ), iter.next() );
		assertEquals( a.getDocumentById( 129 ), iter.next() );
		assertFalse( iter.hasNext() );
	}


}
